@@ -34,28 +34,29 @@ import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 
 import javax.naming.NamingException;
-import javax.servlet.Filter;
-import javax.servlet.FilterRegistration;
-import javax.servlet.RequestDispatcher;
-import javax.servlet.Servlet;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletContextAttributeEvent;
-import javax.servlet.ServletContextAttributeListener;
-import javax.servlet.ServletContextListener;
-import javax.servlet.ServletException;
-import javax.servlet.ServletRegistration;
-import javax.servlet.ServletRegistration.Dynamic;
-import javax.servlet.ServletRequestAttributeListener;
-import javax.servlet.ServletRequestListener;
-import javax.servlet.ServletSecurityElement;
-import javax.servlet.SessionCookieConfig;
-import javax.servlet.SessionTrackingMode;
-import javax.servlet.annotation.ServletSecurity;
-import javax.servlet.descriptor.JspConfigDescriptor;
-import javax.servlet.http.HttpServletMapping;
-import javax.servlet.http.HttpSessionAttributeListener;
-import javax.servlet.http.HttpSessionIdListener;
-import javax.servlet.http.HttpSessionListener;
+
+import jakarta.servlet.Filter;
+import jakarta.servlet.FilterRegistration;
+import jakarta.servlet.RequestDispatcher;
+import jakarta.servlet.Servlet;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletContextAttributeEvent;
+import jakarta.servlet.ServletContextAttributeListener;
+import jakarta.servlet.ServletContextListener;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.ServletRegistration;
+import jakarta.servlet.ServletRegistration.Dynamic;
+import jakarta.servlet.ServletRequestAttributeListener;
+import jakarta.servlet.ServletRequestListener;
+import jakarta.servlet.ServletSecurityElement;
+import jakarta.servlet.SessionCookieConfig;
+import jakarta.servlet.SessionTrackingMode;
+import jakarta.servlet.annotation.ServletSecurity;
+import jakarta.servlet.descriptor.JspConfigDescriptor;
+import jakarta.servlet.http.HttpServletMapping;
+import jakarta.servlet.http.HttpSessionAttributeListener;
+import jakarta.servlet.http.HttpSessionIdListener;
+import jakarta.servlet.http.HttpSessionListener;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
@@ -80,35 +81,22 @@ import org.apache.tomcat.util.res.StringManager;
 
 
 /**
- * Standard implementation of <code>ServletContext</code> that represents a web application's execution environment. An
- * instance of this class is associated with each instance of <code>StandardContext</code>.
+ * Standard implementation of <code>ServletContext</code> that represents
+ * a web application's execution environment.  An instance of this class is
+ * associated with each instance of <code>StandardContext</code>.
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
  */
 public class ApplicationContext implements ServletContext {
 
-    protected static final boolean STRICT_SERVLET_COMPLIANCE;
-
-    protected static final boolean GET_RESOURCE_REQUIRE_SLASH;
-
-    static {
-        STRICT_SERVLET_COMPLIANCE = Globals.STRICT_SERVLET_COMPLIANCE;
-
-        String requireSlash =
-                System.getProperty("org.apache.catalina.core.ApplicationContext.GET_RESOURCE_REQUIRE_SLASH");
-        if (requireSlash == null) {
-            GET_RESOURCE_REQUIRE_SLASH = STRICT_SERVLET_COMPLIANCE;
-        } else {
-            GET_RESOURCE_REQUIRE_SLASH = Boolean.parseBoolean(requireSlash);
-        }
-    }
-
 
     // ----------------------------------------------------------- Constructors
 
+
     /**
-     * Construct a new instance of this class, associated with the specified Context instance.
+     * Construct a new instance of this class, associated with the specified
+     * Context instance.
      *
      * @param context The associated Context instance
      */
@@ -189,7 +177,7 @@ public class ApplicationContext implements ServletContext {
     /**
      * Session Cookie config
      */
-    private final SessionCookieConfig sessionCookieConfig;
+    private SessionCookieConfig sessionCookieConfig;
 
     /**
      * Session tracking modes
@@ -199,8 +187,9 @@ public class ApplicationContext implements ServletContext {
     private Set<SessionTrackingMode> supportedSessionTrackingModes = null;
 
     /**
-     * Flag that indicates if a new {@link ServletContextListener} may be added to the application. Once the first
-     * {@link ServletContextListener} is called, no more may be added.
+     * Flag that indicates if a new {@link ServletContextListener} may be added
+     * to the application. Once the first {@link ServletContextListener} is
+     * called, no more may be added.
      */
     private boolean newServletContextListenerAllowed = true;
 
@@ -228,7 +217,7 @@ public class ApplicationContext implements ServletContext {
             return null;
         }
 
-        Context child;
+        Context child = null;
         try {
             // Look for an exact match
             Container host = context.getParent();
@@ -246,9 +235,9 @@ public class ApplicationContext implements ServletContext {
                     uri = uri.substring(0, i);
                 }
                 // Note: This could be more efficient with a dedicated Mapper
-                // method but such an implementation would require some
-                // refactoring of the Mapper to avoid copy/paste of
-                // existing code.
+                //       method but such an implementation would require some
+                //       refactoring of the Mapper to avoid copy/paste of
+                //       existing code.
                 MessageBytes hostMB = MessageBytes.newInstance();
                 hostMB.setString(host.getName());
 
@@ -291,7 +280,8 @@ public class ApplicationContext implements ServletContext {
     public String getInitParameter(final String name) {
         // Special handling for XML settings as the context setting must
         // always override anything that might have been set by an application.
-        if (Globals.JASPER_XML_VALIDATION_TLD_INIT_PARAM.equals(name) && context.getTldValidation()) {
+        if (Globals.JASPER_XML_VALIDATION_TLD_INIT_PARAM.equals(name) &&
+                context.getTldValidation()) {
             return "true";
         }
         if (Globals.JASPER_XML_BLOCK_EXTERNAL_INIT_PARAM.equals(name)) {
@@ -331,6 +321,12 @@ public class ApplicationContext implements ServletContext {
     }
 
 
+    /**
+     * Return the MIME type of the specified file, or <code>null</code> if
+     * the MIME type cannot be determined.
+     *
+     * @param file Filename for which to identify a MIME type
+     */
     @Override
     public String getMimeType(String file) {
 
@@ -342,7 +338,7 @@ public class ApplicationContext implements ServletContext {
             return null;
         }
         String extension = file.substring(period + 1);
-        if (extension.isEmpty()) {
+        if (extension.length() < 1) {
             return null;
         }
         return context.findMimeMapping(extension);
@@ -350,6 +346,12 @@ public class ApplicationContext implements ServletContext {
     }
 
 
+    /**
+     * Return a <code>RequestDispatcher</code> object that acts as a
+     * wrapper for the named servlet.
+     *
+     * @param name Name of the servlet for which a dispatcher is requested
+     */
     @Override
     public RequestDispatcher getNamedDispatcher(String name) {
 
@@ -384,7 +386,8 @@ public class ApplicationContext implements ServletContext {
             return null;
         }
         if (!path.startsWith("/")) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.requestDispatcher.iae", path));
+            throw new IllegalArgumentException(
+                    sm.getString("applicationContext.requestDispatcher.iae", path));
         }
 
         // Same processing order as InputBuffer / CoyoteAdapter
@@ -400,32 +403,37 @@ public class ApplicationContext implements ServletContext {
             queryString = null;
         }
 
-        // From this point, the removal of path parameters, decoding and normalization is only for mapping purposes.
         // Remove path parameters
-        String uriToMap = org.apache.catalina.util.RequestUtil.stripPathParams(uri, null);
-
-        // Decode only if the uri derived from the provided path is expected to be encoded
-        if (getContext().getDispatchersUseEncodedPaths()) {
-            uriToMap = UDecoder.URLDecode(uriToMap, StandardCharsets.UTF_8, context.getEncodedSolidusHandlingEnum(),
-                    context.getEncodedReverseSolidusHandlingEnum());
-        }
+        String uriNoParams = stripPathParams(uri);
 
         // Then normalize
-        uriToMap = RequestUtil.normalize(uriToMap);
-        if (uriToMap == null) {
-            getContext().getLogger().warn(sm.getString("applicationContext.illegalDispatchPath", path),
-                    new IllegalArgumentException());
+        String normalizedUri = RequestUtil.normalize(uriNoParams);
+        if (normalizedUri == null) {
             return null;
         }
 
-        /*
-         * uri is passed to the constructor for ApplicationDispatcher and is ultimately used as the value for
-         * getRequestURI() which returns encoded values. getContextPath() returns a decoded value. uri may be encoded or
-         * not. Need to prepend the context path to uri and ensure the result is correctly encoded.
-         */
+        // Mapping is against the normalized uri
+
         if (getContext().getDispatchersUseEncodedPaths()) {
+            // Decode
+            String decodedUri = UDecoder.URLDecode(normalizedUri, StandardCharsets.UTF_8);
+
+            // Security check to catch attempts to encode /../ sequences
+            normalizedUri = RequestUtil.normalize(decodedUri);
+            if (!decodedUri.equals(normalizedUri)) {
+                getContext().getLogger().warn(
+                        sm.getString("applicationContext.illegalDispatchPath", path),
+                        new IllegalArgumentException());
+                return null;
+            }
+
+            // URI needs to include the context path
             uri = URLEncoder.DEFAULT.encode(getContextPath(), StandardCharsets.UTF_8) + uri;
         } else {
+            // uri is passed to the constructor for ApplicationDispatcher and is
+            // ultimately used as the value for getRequestURI() which returns
+            // encoded values. Therefore, since the value passed in for path
+            // was decoded, encode uri here.
             uri = URLEncoder.DEFAULT.encode(getContextPath() + uri, StandardCharsets.UTF_8);
         }
 
@@ -442,11 +450,10 @@ public class ApplicationContext implements ServletContext {
 
         try {
             // Map the URI
-            uriMB.setChars(MessageBytes.EMPTY_CHAR_ARRAY, 0, 0);
             CharChunk uriCC = uriMB.getCharChunk();
             try {
                 uriCC.append(context.getPath());
-                uriCC.append(uriToMap);
+                uriCC.append(normalizedUri);
                 service.getMapper().map(context, uriMB, mappingData);
                 if (mappingData.wrapper == null) {
                     return null;
@@ -463,7 +470,8 @@ public class ApplicationContext implements ServletContext {
             HttpServletMapping mapping = new ApplicationMapping(mappingData).getHttpServletMapping();
 
             // Construct a RequestDispatcher to process this request
-            return new ApplicationDispatcher(wrapper, uri, wrapperPath, pathInfo, queryString, mapping, null);
+            return new ApplicationDispatcher(wrapper, uri, wrapperPath, pathInfo,
+                    queryString, mapping, null);
         } finally {
             // Recycle thread local data at the end of the request so references
             // are not held to a completed request as there is potential for
@@ -477,13 +485,42 @@ public class ApplicationContext implements ServletContext {
     }
 
 
+    // Package private to facilitate testing
+    static String stripPathParams(String input) {
+        // Shortcut
+        if (input.indexOf(';') < 0) {
+            return input;
+        }
+
+        StringBuilder sb = new StringBuilder(input.length());
+        int pos = 0;
+        int limit = input.length();
+        while (pos < limit) {
+            int nextSemiColon = input.indexOf(';', pos);
+            if (nextSemiColon < 0) {
+                nextSemiColon = limit;
+            }
+            sb.append(input.substring(pos, nextSemiColon));
+            int followingSlash = input.indexOf('/', nextSemiColon);
+            if (followingSlash < 0) {
+                pos = limit;
+            } else {
+                pos = followingSlash;
+            }
+        }
+
+        return sb.toString();
+    }
+
+
     @Override
     public URL getResource(String path) throws MalformedURLException {
 
-        String validatedPath = validateResourcePath(path, !GET_RESOURCE_REQUIRE_SLASH);
+        String validatedPath = validateResourcePath(path, !context.getContextGetResourceRequiresSlash());
 
         if (validatedPath == null) {
-            throw new MalformedURLException(sm.getString("applicationContext.requestDispatcher.iae", path));
+            throw new MalformedURLException(
+                    sm.getString("applicationContext.requestDispatcher.iae", path));
         }
 
         WebResourceRoot resources = context.getResources();
@@ -498,7 +535,7 @@ public class ApplicationContext implements ServletContext {
     @Override
     public InputStream getResourceAsStream(String path) {
 
-        String validatedPath = validateResourcePath(path, !GET_RESOURCE_REQUIRE_SLASH);
+        String validatedPath = validateResourcePath(path, !context.getContextGetResourceRequiresSlash());
 
         if (validatedPath == null) {
             return null;
@@ -514,7 +551,8 @@ public class ApplicationContext implements ServletContext {
 
 
     /*
-     * Returns null if the input path is not valid or a path that will be acceptable to resources.getResource().
+     * Returns null if the input path is not valid or a path that will be
+     * acceptable to resources.getResource().
      */
     private String validateResourcePath(String path, boolean addMissingInitialSlash) {
         if (path == null) {
@@ -541,7 +579,7 @@ public class ApplicationContext implements ServletContext {
             return null;
         }
         if (!path.startsWith("/")) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.resourcePaths.iae", path));
+            throw new IllegalArgumentException (sm.getString("applicationContext.resourcePaths.iae", path));
         }
 
         WebResourceRoot resources = context.getResources();
@@ -608,22 +646,25 @@ public class ApplicationContext implements ServletContext {
     @Override
     public void removeAttribute(String name) {
 
+        Object value = null;
+
         // Remove the specified attribute
         // Check for read only attribute
-        if (readOnlyAttributes.containsKey(name)) {
+        if (readOnlyAttributes.containsKey(name)){
             return;
         }
-        Object value = attributes.remove(name);
+        value = attributes.remove(name);
         if (value == null) {
             return;
         }
 
         // Notify interested application event listeners
-        Object[] listeners = context.getApplicationEventListeners();
-        if (listeners == null || listeners.length == 0) {
+        Object listeners[] = context.getApplicationEventListeners();
+        if ((listeners == null) || (listeners.length == 0)) {
             return;
         }
-        ServletContextAttributeEvent event = new ServletContextAttributeEvent(context.getServletContext(), name, value);
+        ServletContextAttributeEvent event = new ServletContextAttributeEvent(
+                context.getServletContext(), name, value);
         for (Object obj : listeners) {
             if (!(obj instanceof ServletContextAttributeListener)) {
                 continue;
@@ -636,6 +677,7 @@ public class ApplicationContext implements ServletContext {
             } catch (Throwable t) {
                 ExceptionUtils.handleThrowable(t);
                 context.fireContainerEvent("afterContextAttributeRemoved", listener);
+                // FIXME - should we do anything besides log these?
                 log(sm.getString("applicationContext.attributeEvent"), t);
             }
         }
@@ -665,11 +707,11 @@ public class ApplicationContext implements ServletContext {
         boolean replaced = oldValue != null;
 
         // Notify interested application event listeners
-        Object[] listeners = context.getApplicationEventListeners();
-        if (listeners == null || listeners.length == 0) {
+        Object listeners[] = context.getApplicationEventListeners();
+        if ((listeners == null) || (listeners.length == 0)) {
             return;
         }
-        ServletContextAttributeEvent event;
+        ServletContextAttributeEvent event = null;
         if (replaced) {
             event = new ServletContextAttributeEvent(context.getServletContext(), name, oldValue);
         } else {
@@ -698,6 +740,7 @@ public class ApplicationContext implements ServletContext {
                 } else {
                     context.fireContainerEvent("afterContextAttributeAdded", listener);
                 }
+                // FIXME - should we do anything besides log these?
                 log(sm.getString("applicationContext.attributeEvent"), t);
             }
         }
@@ -717,20 +760,26 @@ public class ApplicationContext implements ServletContext {
 
 
     @Override
-    public FilterRegistration.Dynamic addFilter(String filterName, Class<? extends Filter> filterClass) {
+    public FilterRegistration.Dynamic addFilter(String filterName,
+            Class<? extends Filter> filterClass) {
         return addFilter(filterName, filterClass.getName(), null);
     }
 
 
-    private FilterRegistration.Dynamic addFilter(String filterName, String filterClass, Filter filter)
-            throws IllegalStateException {
+    private FilterRegistration.Dynamic addFilter(String filterName,
+            String filterClass, Filter filter) throws IllegalStateException {
 
-        if (filterName == null || filterName.isEmpty()) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.invalidFilterName", filterName));
+        if (filterName == null || filterName.equals("")) {
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.invalidFilterName", filterName));
         }
 
-        // TODO Spec breaking enhancement to ignore this restriction
-        checkState("applicationContext.addFilter.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            //TODO Spec breaking enhancement to ignore this restriction
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.addFilter.ise",
+                            getContextPath()));
+        }
 
         FilterDef filterDef = context.findFilterDef(filterName);
 
@@ -741,7 +790,8 @@ public class ApplicationContext implements ServletContext {
             filterDef.setFilterName(filterName);
             context.addFilterDef(filterDef);
         } else {
-            if (filterDef.getFilterName() != null && filterDef.getFilterClass() != null) {
+            if (filterDef.getFilterName() != null &&
+                    filterDef.getFilterClass() != null) {
                 return null;
             }
         }
@@ -795,7 +845,8 @@ public class ApplicationContext implements ServletContext {
 
 
     @Override
-    public ServletRegistration.Dynamic addServlet(String servletName, Class<? extends Servlet> servletClass) {
+    public ServletRegistration.Dynamic addServlet(String servletName,
+            Class<? extends Servlet> servletClass) {
         return addServlet(servletName, servletClass.getName(), null, null);
     }
 
@@ -805,14 +856,15 @@ public class ApplicationContext implements ServletContext {
 
         // jspName is validated in addServlet()
         if (jspFile == null || !jspFile.startsWith("/")) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.addJspFile.iae", jspFile));
+            throw new IllegalArgumentException(
+                    sm.getString("applicationContext.addJspFile.iae", jspFile));
         }
 
+        String jspServletClassName = null;
         Map<String,String> jspFileInitParams = new HashMap<>();
 
         Wrapper jspServlet = (Wrapper) context.findChild("jsp");
 
-        String jspServletClassName;
         if (jspServlet == null) {
             // No JSP servlet currently defined.
             // Use default JSP Servlet class name
@@ -835,15 +887,20 @@ public class ApplicationContext implements ServletContext {
     }
 
 
-    private ServletRegistration.Dynamic addServlet(String servletName, String servletClass, Servlet servlet,
-            Map<String,String> initParams) throws IllegalStateException {
+    private ServletRegistration.Dynamic addServlet(String servletName, String servletClass,
+            Servlet servlet, Map<String,String> initParams) throws IllegalStateException {
 
-        if (servletName == null || servletName.isEmpty()) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.invalidServletName", servletName));
+        if (servletName == null || servletName.equals("")) {
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.invalidServletName", servletName));
         }
 
-        // TODO Spec breaking enhancement to ignore this restriction
-        checkState("applicationContext.addServlet.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            //TODO Spec breaking enhancement to ignore this restriction
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.addServlet.ise",
+                            getContextPath()));
+        }
 
         Wrapper wrapper = (Wrapper) context.findChild(servletName);
 
@@ -854,7 +911,8 @@ public class ApplicationContext implements ServletContext {
             wrapper.setName(servletName);
             context.addChild(wrapper);
         } else {
-            if (wrapper.getName() != null && wrapper.getServletClass() != null) {
+            if (wrapper.getName() != null &&
+                    wrapper.getServletClass() != null) {
                 if (wrapper.isOverridable()) {
                     wrapper.setOverridable(false);
                 } else {
@@ -879,12 +937,13 @@ public class ApplicationContext implements ServletContext {
         }
 
         if (initParams != null) {
-            for (Map.Entry<String,String> initParam : initParams.entrySet()) {
+            for (Map.Entry<String, String> initParam: initParams.entrySet()) {
                 wrapper.addInitParameter(initParam.getKey(), initParam.getValue());
             }
         }
 
-        ServletRegistration.Dynamic registration = new ApplicationServletRegistration(wrapper, context);
+        ServletRegistration.Dynamic registration =
+                new ApplicationServletRegistration(wrapper, context);
         if (annotation != null) {
             registration.setServletSecurity(new ServletSecurityElement(annotation));
         }
@@ -935,7 +994,7 @@ public class ApplicationContext implements ServletContext {
             supportedSessionTrackingModes.add(SessionTrackingMode.COOKIE);
         }
 
-        // SSL not enabled by default as it can only be used on its own
+        // SSL not enabled by default as it can only used on its own
         // Context > Host > Engine > Service
         Connector[] connectors = service.findConnectors();
         // Need at least one SSL enabled connector to use the SSL session ID.
@@ -966,21 +1025,27 @@ public class ApplicationContext implements ServletContext {
     @Override
     public void setSessionTrackingModes(Set<SessionTrackingMode> sessionTrackingModes) {
 
-        checkState("applicationContext.setSessionTracking.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.setSessionTracking.ise",
+                            getContextPath()));
+        }
 
         // Check that only supported tracking modes have been requested
         for (SessionTrackingMode sessionTrackingMode : sessionTrackingModes) {
             if (!supportedSessionTrackingModes.contains(sessionTrackingMode)) {
-                throw new IllegalArgumentException(sm.getString("applicationContext.setSessionTracking.iae.invalid",
+                throw new IllegalArgumentException(sm.getString(
+                        "applicationContext.setSessionTracking.iae.invalid",
                         sessionTrackingMode.toString(), getContextPath()));
             }
         }
 
-        // Check SSL has not been configured with anything else
+        // Check SSL has not be configured with anything else
         if (sessionTrackingModes.contains(SessionTrackingMode.SSL)) {
             if (sessionTrackingModes.size() > 1) {
-                throw new IllegalArgumentException(
-                        sm.getString("applicationContext.setSessionTracking.iae.ssl", getContextPath()));
+                throw new IllegalArgumentException(sm.getString(
+                        "applicationContext.setSessionTracking.iae.ssl",
+                        getContextPath()));
             }
         }
 
@@ -994,7 +1059,12 @@ public class ApplicationContext implements ServletContext {
         if (name == null) {
             throw new NullPointerException(sm.getString("applicationContext.setAttribute.namenull"));
         }
-        checkState("applicationContext.setInitParam.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.setInitParam.ise",
+                            getContextPath()));
+        }
+
         return parameters.putIfAbsent(name, value) == null;
     }
 
@@ -1005,8 +1075,9 @@ public class ApplicationContext implements ServletContext {
         try {
             listener = createListener(listenerClass);
         } catch (ServletException e) {
-            throw new IllegalArgumentException(
-                    sm.getString("applicationContext.addListener.iae.init", listenerClass.getName()), e);
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.addListener.iae.init",
+                    listenerClass.getName()), e);
         }
         addListener(listener);
     }
@@ -1020,8 +1091,9 @@ public class ApplicationContext implements ServletContext {
                 Object obj = context.getInstanceManager().newInstance(className);
 
                 if (!(obj instanceof EventListener)) {
-                    throw new IllegalArgumentException(
-                            sm.getString("applicationContext.addListener.iae.wrongType", className));
+                    throw new IllegalArgumentException(sm.getString(
+                            "applicationContext.addListener.iae.wrongType",
+                            className));
                 }
 
                 EventListener listener = (EventListener) obj;
@@ -1029,9 +1101,13 @@ public class ApplicationContext implements ServletContext {
             }
         } catch (InvocationTargetException e) {
             ExceptionUtils.handleThrowable(e.getCause());
-            throw new IllegalArgumentException(sm.getString("applicationContext.addListener.iae.cnfe", className), e);
-        } catch (ReflectiveOperationException | NamingException e) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.addListener.iae.cnfe", className), e);
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.addListener.iae.cnfe", className),
+                    e);
+        } catch (ReflectiveOperationException| NamingException e) {
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.addListener.iae.cnfe", className),
+                    e);
         }
 
     }
@@ -1039,11 +1115,17 @@ public class ApplicationContext implements ServletContext {
 
     @Override
     public <T extends EventListener> void addListener(T t) {
-        checkState("applicationContext.addListener.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.addListener.ise",
+                            getContextPath()));
+        }
 
         boolean match = false;
-        if (t instanceof ServletContextAttributeListener || t instanceof ServletRequestListener ||
-                t instanceof ServletRequestAttributeListener || t instanceof HttpSessionIdListener ||
+        if (t instanceof ServletContextAttributeListener ||
+                t instanceof ServletRequestListener ||
+                t instanceof ServletRequestAttributeListener ||
+                t instanceof HttpSessionIdListener ||
                 t instanceof HttpSessionAttributeListener) {
             context.addApplicationEventListener(t);
             match = true;
@@ -1062,28 +1144,35 @@ public class ApplicationContext implements ServletContext {
         }
 
         if (t instanceof ServletContextListener) {
-            throw new IllegalArgumentException(
-                    sm.getString("applicationContext.addListener.iae.sclNotAllowed", t.getClass().getName()));
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.addListener.iae.sclNotAllowed",
+                    t.getClass().getName()));
         } else {
-            throw new IllegalArgumentException(
-                    sm.getString("applicationContext.addListener.iae.wrongType", t.getClass().getName()));
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.addListener.iae.wrongType",
+                    t.getClass().getName()));
         }
     }
 
 
     @Override
-    public <T extends EventListener> T createListener(Class<T> c) throws ServletException {
+    public <T extends EventListener> T createListener(Class<T> c)
+            throws ServletException {
         try {
             @SuppressWarnings("unchecked")
             T listener = (T) context.getInstanceManager().newInstance(c);
-            if (listener instanceof ServletContextListener || listener instanceof ServletContextAttributeListener ||
-                    listener instanceof ServletRequestListener || listener instanceof ServletRequestAttributeListener ||
-                    listener instanceof HttpSessionListener || listener instanceof HttpSessionIdListener ||
+            if (listener instanceof ServletContextListener ||
+                    listener instanceof ServletContextAttributeListener ||
+                    listener instanceof ServletRequestListener ||
+                    listener instanceof ServletRequestAttributeListener ||
+                    listener instanceof HttpSessionListener ||
+                    listener instanceof HttpSessionIdListener ||
                     listener instanceof HttpSessionAttributeListener) {
                 return listener;
             }
-            throw new IllegalArgumentException(
-                    sm.getString("applicationContext.addListener.iae.wrongType", listener.getClass().getName()));
+            throw new IllegalArgumentException(sm.getString(
+                    "applicationContext.addListener.iae.wrongType",
+                    listener.getClass().getName()));
         } catch (InvocationTargetException e) {
             ExceptionUtils.handleThrowable(e.getCause());
             throw new ServletException(e);
@@ -1096,16 +1185,24 @@ public class ApplicationContext implements ServletContext {
     @Override
     public void declareRoles(String... roleNames) {
 
-        // TODO Spec breaking enhancement to ignore this restriction
-        checkState("applicationContext.addRole.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            //TODO Spec breaking enhancement to ignore this restriction
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.addRole.ise",
+                            getContextPath()));
+        }
 
         if (roleNames == null) {
-            throw new IllegalArgumentException(sm.getString("applicationContext.roles.iae", getContextPath()));
+            throw new IllegalArgumentException(
+                    sm.getString("applicationContext.roles.iae",
+                            getContextPath()));
         }
 
         for (String role : roleNames) {
             if (role == null || role.isEmpty()) {
-                throw new IllegalArgumentException(sm.getString("applicationContext.role.iae", getContextPath()));
+                throw new IllegalArgumentException(
+                        sm.getString("applicationContext.role.iae",
+                                getContextPath()));
             }
             context.addSecurityRole(role);
         }
@@ -1125,7 +1222,8 @@ public class ApplicationContext implements ServletContext {
                 parent = parent.getParent();
             }
             if (parent == null) {
-                System.getSecurityManager().checkPermission(new RuntimePermission("getClassLoader"));
+                System.getSecurityManager().checkPermission(
+                        new RuntimePermission("getClassLoader"));
             }
         }
 
@@ -1146,12 +1244,13 @@ public class ApplicationContext implements ServletContext {
 
 
     @Override
-    public Map<String,? extends FilterRegistration> getFilterRegistrations() {
-        Map<String,ApplicationFilterRegistration> result = new HashMap<>();
+    public Map<String, ? extends FilterRegistration> getFilterRegistrations() {
+        Map<String, ApplicationFilterRegistration> result = new HashMap<>();
 
         FilterDef[] filterDefs = context.findFilterDefs();
         for (FilterDef filterDef : filterDefs) {
-            result.put(filterDef.getFilterName(), new ApplicationFilterRegistration(filterDef, context));
+            result.put(filterDef.getFilterName(),
+                    new ApplicationFilterRegistration(filterDef, context));
         }
 
         return result;
@@ -1165,12 +1264,14 @@ public class ApplicationContext implements ServletContext {
 
 
     @Override
-    public Map<String,? extends ServletRegistration> getServletRegistrations() {
-        Map<String,ApplicationServletRegistration> result = new HashMap<>();
+    public Map<String, ? extends ServletRegistration> getServletRegistrations() {
+        Map<String, ApplicationServletRegistration> result = new HashMap<>();
 
         Container[] wrappers = context.findChildren();
         for (Container wrapper : wrappers) {
-            result.put(wrapper.getName(), new ApplicationServletRegistration((Wrapper) wrapper, context));
+            result.put(wrapper.getName(),
+                    new ApplicationServletRegistration(
+                            (Wrapper) wrapper, context));
         }
 
         return result;
@@ -1194,7 +1295,12 @@ public class ApplicationContext implements ServletContext {
 
     @Override
     public void setSessionTimeout(int sessionTimeout) {
-        checkState("applicationContext.setSessionTimeout.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.setSessionTimeout.ise",
+                            getContextPath()));
+        }
+
         context.setSessionTimeout(sessionTimeout);
     }
 
@@ -1207,7 +1313,12 @@ public class ApplicationContext implements ServletContext {
 
     @Override
     public void setRequestCharacterEncoding(String encoding) {
-        checkState("applicationContext.setRequestEncoding.ise");
+        if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.setRequestEncoding.ise",
+                            getContextPath()));
+        }
+
         context.setRequestCharacterEncoding(encoding);
     }
 
@@ -1220,15 +1331,13 @@ public class ApplicationContext implements ServletContext {
 
     @Override
     public void setResponseCharacterEncoding(String encoding) {
-        checkState("applicationContext.setResponseEncoding.ise");
-        context.setResponseCharacterEncoding(encoding);
-    }
-
-
-    private void checkState(String messageKey) {
         if (!context.getState().equals(LifecycleState.STARTING_PREP)) {
-            throw new IllegalStateException(sm.getString(messageKey, getContextPath()));
+            throw new IllegalStateException(
+                    sm.getString("applicationContext.setResponseEncoding.ise",
+                            getContextPath()));
         }
+
+        context.setResponseCharacterEncoding(encoding);
     }
 
 
@@ -1279,14 +1388,15 @@ public class ApplicationContext implements ServletContext {
     }
 
     /**
-     * Internal class used as thread-local storage when doing path mapping during dispatch.
+     * Internal class used as thread-local storage when doing path
+     * mapping during dispatch.
      */
     private static final class DispatchData {
 
         public MessageBytes uriMB;
         public MappingData mappingData;
 
-        DispatchData() {
+        public DispatchData() {
             uriMB = MessageBytes.newInstance();
             CharChunk uriCC = uriMB.getCharChunk();
             uriCC.setLimit(-1);
