@@ -26,22 +26,26 @@ import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
 
-import javax.el.ELException;
-import javax.el.MethodNotFoundException;
+import jakarta.el.ELException;
+import jakarta.el.MethodNotFoundException;
 
 import org.apache.el.lang.ELSupport;
 import org.apache.el.lang.EvaluationContext;
 
+
 /**
- * Utilities for Managing Serialization and Reflection.
+ * Utilities for Managing Serialization and Reflection
+ *
+ * @author Jacob Hookom [jacob@hookom.net]
  */
 public class ReflectionUtil {
 
-    protected static final String[] PRIMITIVE_NAMES =
-            new String[] { "boolean", "byte", "char", "double", "float", "int", "long", "short", "void" };
+    protected static final String[] PRIMITIVE_NAMES = new String[] { "boolean",
+            "byte", "char", "double", "float", "int", "long", "short", "void" };
 
-    protected static final Class<?>[] PRIMITIVES = new Class[] { boolean.class, byte.class, char.class, double.class,
-            float.class, int.class, long.class, short.class, Void.TYPE };
+    protected static final Class<?>[] PRIMITIVES = new Class[] { boolean.class,
+            byte.class, char.class, double.class, float.class, int.class,
+            long.class, short.class, Void.TYPE };
 
     private ReflectionUtil() {
         super();
@@ -76,12 +80,10 @@ public class ReflectionUtil {
 
     /**
      * Converts an array of Class names to Class types.
-     *
-     * @param s The array of class names
-     *
-     * @return An array of Class instance where the element at index i in the result is an instance of the class with
-     *             the name at index i in the input
-     *
+     * @param s  The array of class names
+     * @return An array of Class instance where the element at index i in the
+     *         result is an instance of the class with the name at index i in
+     *         the input
      * @throws ClassNotFoundException If a class of a given name cannot be found
      */
     public static Class<?>[] toTypeArray(String[] s) throws ClassNotFoundException {
@@ -97,11 +99,9 @@ public class ReflectionUtil {
 
     /**
      * Converts an array of Class types to Class names.
-     *
      * @param c The array of class instances
-     *
-     * @return An array of Class names where the element at index i in the result is the name of the class instance at
-     *             index i in the input
+     * @return An array of Class names where the element at index i in the
+     *         result is the name of the class instance at index i in the input
      */
     public static String[] toTypeNameArray(Class<?>[] c) {
         if (c == null) {
@@ -116,30 +116,32 @@ public class ReflectionUtil {
 
     /**
      * Returns a method based on the criteria.
-     *
-     * @param ctx         the context in which the expression is being evaluated
-     * @param base        the object that owns the method
-     * @param property    the name of the method
-     * @param paramTypes  the parameter types to use
+     * @param ctx the context in which the expression is being evaluated
+     * @param base the object that owns the method
+     * @param property the name of the method
+     * @param paramTypes the parameter types to use
      * @param paramValues the parameter values
-     *
      * @return the method specified
-     *
-     * @throws MethodNotFoundException If a method cannot be found that matches the given criteria
+     * @throws MethodNotFoundException If a method cannot be found that matches
+     *         the given criteria
      */
     /*
-     * This class duplicates code in javax.el.Util. When making changes keep the code in sync.
+     * This class duplicates code in jakarta.el.Util. When making changes keep
+     * the code in sync.
      */
     @SuppressWarnings("null")
-    public static Method getMethod(EvaluationContext ctx, Object base, Object property, Class<?>[] paramTypes,
-            Object[] paramValues) throws MethodNotFoundException {
+    public static Method getMethod(EvaluationContext ctx, Object base, Object property,
+            Class<?>[] paramTypes, Object[] paramValues)
+            throws MethodNotFoundException {
 
         if (base == null || property == null) {
-            throw new MethodNotFoundException(
-                    MessageFactory.get("error.method.notfound", base, property, paramString(paramTypes)));
+            throw new MethodNotFoundException(MessageFactory.get(
+                    "error.method.notfound", base, property,
+                    paramString(paramTypes)));
         }
 
-        String methodName = (property instanceof String) ? (String) property : property.toString();
+        String methodName = (property instanceof String) ? (String) property
+                : property.toString();
 
         int paramCount;
         if (paramTypes == null) {
@@ -148,27 +150,7 @@ public class ReflectionUtil {
             paramCount = paramTypes.length;
         }
 
-        Class<?> clazz = base.getClass();
-
-        // Fast path: when no arguments exist, there can only be one matching method and no need for coercion.
-        if (paramCount == 0) {
-            Method result = null;
-            Throwable t = null;
-            try {
-                Method method = clazz.getMethod(methodName, paramTypes);
-                result = getMethod(clazz, base, method);
-            } catch (NoSuchMethodException | SecurityException e) {
-                // Fall through
-                t = e;
-            }
-            if (result == null) {
-                throw new MethodNotFoundException(
-                        MessageFactory.get("error.method.notfound", base, property, paramString(paramTypes)), t);
-            }
-            return result;
-        }
-
-        Method[] methods = clazz.getMethods();
+        Method[] methods = base.getClass().getMethods();
         Map<Method,MatchResult> candidates = new HashMap<>();
 
         for (Method m : methods) {
@@ -186,16 +168,17 @@ public class ReflectionUtil {
                 // Method has wrong number of parameters
                 continue;
             }
-            if (m.isVarArgs() && paramCount < mParamCount - 1) {
+            if (m.isVarArgs() && paramCount < mParamCount -1) {
                 // Method has wrong number of parameters
                 continue;
             }
-            if (m.isVarArgs() && paramCount == mParamCount && paramValues != null && paramValues.length > paramCount &&
-                    !paramTypes[mParamCount - 1].isArray()) {
+            if (m.isVarArgs() && paramCount == mParamCount && paramValues != null &&
+                    paramValues.length > paramCount && !paramTypes[mParamCount -1].isArray()) {
                 // Method arguments don't match
                 continue;
             }
-            if (m.isVarArgs() && paramCount > mParamCount && paramValues != null && paramValues.length != paramCount) {
+            if (m.isVarArgs() && paramCount > mParamCount && paramValues != null &&
+                    paramValues.length != paramCount) {
                 // Might match a different varargs method
                 continue;
             }
@@ -269,16 +252,11 @@ public class ReflectionUtil {
             // If a method is found where every parameter matches exactly,
             // and no vars args are present, return it
             if (exactMatch == paramCount && varArgsMatch == 0) {
-                Method result = getMethod(clazz, base, m);
-                if (result == null) {
-                    throw new MethodNotFoundException(
-                            MessageFactory.get("error.method.notfound", base, property, paramString(paramTypes)));
-                }
-                return result;
+                return getMethod(base.getClass(), base, m);
             }
 
-            candidates.put(m, new MatchResult(m.isVarArgs(), exactMatch, assignableMatch, coercibleMatch, varArgsMatch,
-                    m.isBridge()));
+            candidates.put(m, new MatchResult(
+                    m.isVarArgs(), exactMatch, assignableMatch, coercibleMatch, varArgsMatch, m.isBridge()));
         }
 
         // Look for the method that has the highest number of parameters where
@@ -286,7 +264,7 @@ public class ReflectionUtil {
         MatchResult bestMatch = new MatchResult(true, 0, 0, 0, 0, true);
         Method match = null;
         boolean multiple = false;
-        for (Map.Entry<Method,MatchResult> entry : candidates.entrySet()) {
+        for (Map.Entry<Method, MatchResult> entry : candidates.entrySet()) {
             int cmp = entry.getValue().compareTo(bestMatch);
             if (cmp > 0 || match == null) {
                 bestMatch = entry.getValue();
@@ -308,29 +286,28 @@ public class ReflectionUtil {
             if (match == null) {
                 // If multiple methods have the same matching number of parameters
                 // the match is ambiguous so throw an exception
-                throw new MethodNotFoundException(
-                        MessageFactory.get("error.method.ambiguous", base, property, paramString(paramTypes)));
-            }
+                throw new MethodNotFoundException(MessageFactory.get(
+                        "error.method.ambiguous", base, property,
+                        paramString(paramTypes)));
+                }
         }
 
         // Handle case where no match at all was found
         if (match == null) {
-            throw new MethodNotFoundException(
-                    MessageFactory.get("error.method.notfound", base, property, paramString(paramTypes)));
+            throw new MethodNotFoundException(MessageFactory.get(
+                        "error.method.notfound", base, property,
+                        paramString(paramTypes)));
         }
 
-        Method result = getMethod(clazz, base, match);
-        if (result == null) {
-            throw new MethodNotFoundException(
-                    MessageFactory.get("error.method.notfound", base, property, paramString(paramTypes)));
-        }
-        return result;
+        return getMethod(base.getClass(), base, match);
     }
 
     /*
-     * This class duplicates code in javax.el.Util. When making changes keep the code in sync.
+     * This class duplicates code in jakarta.el.Util. When making changes keep
+     * the code in sync.
      */
-    private static Method resolveAmbiguousMethod(Set<Method> candidates, Class<?>[] paramTypes) {
+    private static Method resolveAmbiguousMethod(Set<Method> candidates,
+            Class<?>[] paramTypes) {
         // Identify which parameter isn't an exact match
         Method m = candidates.iterator().next();
 
@@ -351,11 +328,12 @@ public class ReflectionUtil {
         }
 
         for (Method c : candidates) {
-            if (c.getParameterTypes()[nonMatchIndex] == paramTypes[nonMatchIndex]) {
-                // Methods have different non-matching parameters
-                // Result is ambiguous
-                return null;
-            }
+           if (c.getParameterTypes()[nonMatchIndex] ==
+                   paramTypes[nonMatchIndex]) {
+               // Methods have different non-matching parameters
+               // Result is ambiguous
+               return null;
+           }
         }
 
         // Can't be null
@@ -375,7 +353,8 @@ public class ReflectionUtil {
         if (Number.class.isAssignableFrom(nonMatchClass)) {
             for (Method c : candidates) {
                 Class<?> candidateType = c.getParameterTypes()[nonMatchIndex];
-                if (Number.class.isAssignableFrom(candidateType) || candidateType.isPrimitive()) {
+                if (Number.class.isAssignableFrom(candidateType) ||
+                        candidateType.isPrimitive()) {
                     if (match == null) {
                         match = c;
                     } else {
@@ -392,7 +371,8 @@ public class ReflectionUtil {
 
 
     /*
-     * This class duplicates code in javax.el.Util. When making changes keep the code in sync.
+     * This class duplicates code in jakarta.el.Util. When making changes keep
+     * the code in sync.
      */
     private static boolean isAssignableFrom(Class<?> src, Class<?> target) {
         // src will always be an object
@@ -429,10 +409,12 @@ public class ReflectionUtil {
 
 
     /*
-     * This class duplicates code in javax.el.Util. When making changes keep the code in sync.
+     * This class duplicates code in jakarta.el.Util. When making changes keep
+     * the code in sync.
      */
     private static boolean isCoercibleFrom(EvaluationContext ctx, Object src, Class<?> target) {
-        // TODO: This isn't pretty but it works. Significant refactoring would be required to avoid the exception.
+        // TODO: This isn't pretty but it works. Significant refactoring would
+        //       be required to avoid the exception.
         try {
             ELSupport.coerceToType(ctx, src, target);
         } catch (ELException e) {
@@ -443,17 +425,19 @@ public class ReflectionUtil {
 
 
     /*
-     * This class duplicates code in javax.el.Util. When making changes keep the code in sync.
+     * This class duplicates code in jakarta.el.Util. When making changes keep
+     * the code in sync.
      */
     private static Method getMethod(Class<?> type, Object base, Method m) {
         JreCompat jreCompat = JreCompat.getInstance();
-        if (m == null || (Modifier.isPublic(type.getModifiers()) &&
-                (Modifier.isStatic(m.getModifiers()) && jreCompat.canAccess(null, m) ||
-                        jreCompat.canAccess(base, m)))) {
+        if (m == null ||
+                (Modifier.isPublic(type.getModifiers()) &&
+                        (Modifier.isStatic(m.getModifiers()) && jreCompat.canAccess(null, m)
+                                || jreCompat.canAccess(base, m)))) {
             return m;
         }
         Class<?>[] interfaces = type.getInterfaces();
-        Method mp;
+        Method mp = null;
         for (Class<?> iface : interfaces) {
             try {
                 mp = iface.getMethod(m.getName(), m.getParameterTypes());
@@ -481,7 +465,7 @@ public class ReflectionUtil {
     }
 
 
-    private static String paramString(Class<?>[] types) {
+    private static final String paramString(Class<?>[] types) {
         if (types != null) {
             StringBuilder sb = new StringBuilder();
             for (Class<?> type : types) {
@@ -522,7 +506,8 @@ public class ReflectionUtil {
 
 
     /*
-     * This class duplicates code in javax.el.Util. When making changes keep the code in sync.
+     * This class duplicates code in jakarta.el.Util. When making changes keep
+     * the code in sync.
      */
     private static class MatchResult implements Comparable<MatchResult> {
 
@@ -533,7 +518,7 @@ public class ReflectionUtil {
         private final int varArgsCount;
         private final boolean bridge;
 
-        MatchResult(boolean varArgs, int exactCount, int assignableCount, int coercibleCount, int varArgsCount,
+        public MatchResult(boolean varArgs, int exactCount, int assignableCount, int coercibleCount, int varArgsCount,
                 boolean bridge) {
             this.varArgs = varArgs;
             this.exactCount = exactCount;
@@ -596,13 +581,14 @@ public class ReflectionUtil {
 
         @Override
         public boolean equals(Object o) {
-            return o == this || (null != o && this.getClass().equals(o.getClass()) &&
-                    ((MatchResult) o).getExactCount() == this.getExactCount() &&
-                    ((MatchResult) o).getAssignableCount() == this.getAssignableCount() &&
-                    ((MatchResult) o).getCoercible() == this.getCoercible() &&
-                    ((MatchResult) o).getVarArgsCount() == this.getVarArgsCount() &&
-                    ((MatchResult) o).isVarArgs() == this.isVarArgs() &&
-                    ((MatchResult) o).isBridge() == this.isBridge());
+            return o == this || (null != o &&
+                    this.getClass().equals(o.getClass()) &&
+                    ((MatchResult)o).getExactCount() == this.getExactCount() &&
+                    ((MatchResult)o).getAssignableCount() == this.getAssignableCount() &&
+                    ((MatchResult)o).getCoercible() == this.getCoercible() &&
+                    ((MatchResult)o).getVarArgsCount() == this.getVarArgsCount() &&
+                    ((MatchResult)o).isVarArgs() == this.isVarArgs() &&
+                    ((MatchResult)o).isBridge() == this.isBridge());
         }
 
         @Override
