@@ -94,6 +94,14 @@ public class TestELArithmetic {
         Assert.assertEquals(Long.valueOf(0), result);
     }
 
+    @Test
+    public void testMod03() {
+        ELProcessor processor = new ELProcessor();
+        // Large so BigInteger rather than Long is used internally
+        BigInteger result = (BigInteger) processor.eval("1 % -9999999999999999999");
+        Assert.assertEquals(BigInteger.valueOf(1), result);
+    }
+
     @Test
     public void testUnaryMinus01() {
         ELProcessor processor = new ELProcessor();
